package de.uniulm.inkombio.pareto;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JPanel;

/**
 * This Class wraps the actual SliderComponent and adds the hiding functionality
 * 
 * @author schnattinger
 * 
 */
@SuppressWarnings("serial")
public class HidingSliderComponent extends JPanel {

	private static final String text = "Select optimal data points by weighting the objectives";

	private SliderComponent sliderComponent;
	private JButton button;

	private ParetoPanel paretopanel;

	public HidingSliderComponent(ArrayList<Solution> solutions,
			ScatterPlotPanel scatterplot, ParetoPanel paretopanel) {
		this.paretopanel = paretopanel;
		setLayout(new BorderLayout());

		sliderComponent = new SliderComponent(text, solutions, scatterplot,
				paretopanel.convexhull);
		button = new JButton(text);
		button.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				showMe();
			}
		});

		sliderComponent.setVisible(false);
		add(sliderComponent, BorderLayout.CENTER);
		add(button, BorderLayout.SOUTH);
	}

	/**
	 * Shows the slider component, hides the button, activates the display of
	 * the convex hull, and notifies the sliderComponent
	 */
	protected void showMe() {
		sliderComponent.setVisible(true);
		button.setVisible(false);
		paretopanel.scatterplot.setDisplayConvexHull(true);
		sliderComponent.valueChanged();
	}

	/**
	 * Hides the slider component, shows the button and deactivates the display
	 * of the convex hull
	 */
	public void hideMe() {
		sliderComponent.setVisible(false);
		button.setVisible(true);
		paretopanel.scatterplot.setDisplayConvexHull(false);
	}

	/**
	 * Delegates the setValue of the JSlider.
	 * 
	 * @param value
	 */
	public void setValue(int value) {
		sliderComponent.setValue(value);
	}

}
