package de.uniulm.inkombio.pareto;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;

/**
 * Maybe not the nicest way, but this class offers a method which registers the
 * keystrokes and actions for keyboard input.
 * 
 */
public class KeyStrokeFunctionality {

	@SuppressWarnings("serial")
	public static void addKeyStrokes(final JFrame frame, final JTabbedPane pane) {
		KeyStroke CtrlW = KeyStroke.getKeyStroke(KeyEvent.VK_W,
				InputEvent.CTRL_MASK);
		KeyStroke CtrlQ = KeyStroke.getKeyStroke(KeyEvent.VK_Q,
				InputEvent.CTRL_MASK);
		KeyStroke CtrlPgUp = KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_UP,
				InputEvent.CTRL_MASK);
		KeyStroke CtrlPgDown = KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_DOWN,
				InputEvent.CTRL_MASK);
		KeyStroke CtrlUp = KeyStroke.getKeyStroke(KeyEvent.VK_UP,
				InputEvent.CTRL_MASK);
		KeyStroke CtrlDown = KeyStroke.getKeyStroke(KeyEvent.VK_DOWN,
				InputEvent.CTRL_MASK);
		KeyStroke CtrlHome = KeyStroke.getKeyStroke(KeyEvent.VK_HOME,
				InputEvent.CTRL_MASK);
		KeyStroke CtrlEnd = KeyStroke.getKeyStroke(KeyEvent.VK_END,
				InputEvent.CTRL_MASK);

		InputMap inputmap = frame.getRootPane().getInputMap(
				JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap actionmap = frame.getRootPane().getActionMap();

		inputmap.put(CtrlW, "action_close_tab");
		inputmap.put(CtrlQ, "action_exit_program");
		inputmap.put(CtrlPgUp, "action_tab_left");
		inputmap.put(CtrlPgDown, "action_tab_right");
		inputmap.put(CtrlUp, "action_select_up");
		inputmap.put(CtrlDown, "action_select_down");
		inputmap.put(CtrlHome, "action_select_first");
		inputmap.put(CtrlEnd, "action_select_last");

		actionmap.put("action_close_tab",
				new AbstractAction("action_close_tab") {
					public void actionPerformed(ActionEvent e) {
						int index = pane.getSelectedIndex();
						if (index > -1)
							pane.removeTabAt(index);
					}
				});
		actionmap.put("action_exit_program", new AbstractAction(
				"action_exit_program") {
			public void actionPerformed(ActionEvent e) {
				int ret = JOptionPane.showConfirmDialog(frame,
						"Do you really want to quit?", "Quit",
						JOptionPane.YES_NO_OPTION);
				if (ret == JOptionPane.YES_OPTION) {
					frame.setVisible(false);
					frame.dispose();
					System.exit(0);
				}
			}
		});
		actionmap.put("action_tab_left", new AbstractAction("action_tab_left") {
			public void actionPerformed(ActionEvent arg0) {
				int index = pane.getSelectedIndex() - 1;
				if (index > -1)
					pane.setSelectedIndex(index);
			}
		});
		actionmap.put("action_tab_right",
				new AbstractAction("action_tab_right") {
					public void actionPerformed(ActionEvent e) {
						int index = pane.getSelectedIndex();
						if (index > -1 && index < pane.getTabCount() - 1)
							pane.setSelectedIndex(index + 1);
					}
				});
		actionmap.put("action_select_up",
				new AbstractAction("action_select_up") {
					public void actionPerformed(ActionEvent e) {
						Component panel = pane.getSelectedComponent();
						if (panel != null && panel instanceof ParetoPanel) {
							ParetoPanel paretopanel = (ParetoPanel) panel;
							paretopanel.selectDataPointNodeRelative(1);
						}
					}
				});
		actionmap.put("action_select_down", new AbstractAction(
				"action_select_down") {
			public void actionPerformed(ActionEvent e) {
				Component panel = pane.getSelectedComponent();
				if (panel != null && panel instanceof ParetoPanel) {
					ParetoPanel paretopanel = (ParetoPanel) panel;
					paretopanel.selectDataPointNodeRelative(-1);
				}
			}
		});
		actionmap.put("action_select_first", new AbstractAction(
				"action_select_first") {
			public void actionPerformed(ActionEvent e) {
				Component panel = pane.getSelectedComponent();
				if (panel != null && panel instanceof ParetoPanel) {
					ParetoPanel paretopanel = (ParetoPanel) panel;
					paretopanel
							.selectDataPointNodeRelative(Integer.MAX_VALUE / 2);
				}
			}
		});
		actionmap.put("action_select_last", new AbstractAction(
				"action_select_last") {
			public void actionPerformed(ActionEvent e) {
				Component panel = pane.getSelectedComponent();
				if (panel != null && panel instanceof ParetoPanel) {
					ParetoPanel paretopanel = (ParetoPanel) panel;
					paretopanel
							.selectDataPointNodeRelative(Integer.MIN_VALUE / 2);
				}
			}
		});
	}

}
