#ifndef INCLUDE_ALIGNMENT_HPP
#define INCLUDE_ALIGNMENT_HPP

#include <algorithm>
#include <vector>
#include <string>

#include "rna.hpp"

/* used to reconstruct the alignment 
 */
struct match {
	int type; // 1=unmatched (i/j), 2=unmatched (k/l), 3=matched nucleotide, 4=matched pair
	int i, j, k, l;
	match(const int type, const int n);
	match(const int type, const int i, const int k);
	match(const int type, const int i, const int j, const int k, const int l);
	match(const match &m);

	bool operator<(const match& m) const;
};

struct alignment {
	const std::vector<double> objectives;
	const std::string sequence_a;
	const std::string sequence_b;
	const std::string structure;

	alignment& operator= (const alignment& a);

	alignment(const std::vector<double> &objectives, const std::string &sequence_a, const std::string &sequence_b, const std::string &structure);

	void print_alignment();

	static double energy_of_structure(const std::string &sequence, const std::string &structure);
};

void print_matches(std::vector<match> &result);

// sort matches, descreasing j and l. 
// Since elements like (1,-) and (-,2) are not comparable, we first sort all matches first, and insert the gaps later.
void sort_matches(std::vector<match> &v);

void test_alignment(const std::string &a, const std::string &b);

// construction of the alignemnt from the given matched nucleotides
// here a "matched/unmatched" nucleotide means j/l, and not i/k
alignment make_alignment_jl(RNA &A, RNA &B, std::vector<match> &result, std::vector<double> &objectives);

#endif
