#include <cstring>
#include <iostream>
#include <vector>

#include "alignment.hpp"
#include "rna.hpp"
#include "rna-pareto.hpp"
#include "rna-pareto-affine.hpp"
#include "rna-pareto-normal.hpp"

using namespace std;

void usage(char** argv) {
	cerr << endl << "RNA-Pareto, version " << VERSION_MAJOR << "." << VERSION_MINOR << "." << VERSION_REVISION << endl;
	cerr << "Contact: thomas.schnattinger@uni-ulm.de" << endl;
	cerr << endl << "  usage 1: " << argv[0] << " --direct sequence-1 sequence-2 [--affine]" << endl;
	cerr << endl << "  usage 2: " << argv[0] << " --postscript ps-file-1 ps-file-2 [--affine]" << endl;
	cerr << "  (postscript files must be result of RNAfold -p -noLP)" << endl << endl;
}

int main(int argc, char **argv) {
	if (argc!=4 && argc!=5) {
		usage(argv);
		return 1;
	}
	vector<alignment> alignments;
	if (argc==5 && strcmp(argv[4], "--affine")==0) {
		if (strcmp(argv[1], "--direct")==0) {
			RNA A = RNA::fold_RNA(argv[2]);
			RNA B = RNA::fold_RNA(argv[3]);
			alignments = rna_pareto_affine::find_pareto_alignments(A, B, -2.0, -1.0);
		} else if (strcmp(argv[1], "--postscript")==0) {
			RNA A = RNA::read_RNAfold_file(argv[2]);
			RNA B = RNA::read_RNAfold_file(argv[3]);
			alignments = rna_pareto_affine::find_pareto_alignments(A, B, -2.0, -1.0);
		} else {
			usage(argv);
			return 1;
		}
	} else {
		if (strcmp(argv[1], "--direct")==0) {
			RNA A = RNA::fold_RNA(argv[2]);
			RNA B = RNA::fold_RNA(argv[3]);
			alignments = rna_pareto_normal::find_pareto_alignments(A, B, -3.0);
		} else if (strcmp(argv[1], "--postscript")==0) {
			RNA A = RNA::read_RNAfold_file(argv[2]);
			RNA B = RNA::read_RNAfold_file(argv[3]);
			alignments = rna_pareto_normal::find_pareto_alignments(A, B, -3.0);
		} else {
			usage(argv);
			return 1;
		}
	}
	for (vector<alignment>::iterator it=alignments.begin(); it!=alignments.end(); ++it) {
		it->print_alignment();
	}
	return 0;
}
