#include <cmath>
#include <cstring>
#include <fstream>

#include "rna.hpp"

extern "C" {
	double fill_bppm_RNAfold(const char* seq, int length, double **bppm);
}

RNA::RNA(const std::string &sequence) : length(sequence.length()), sequence(sequence) {
	bppm = new double*[length+1];
	psi = new double*[length+1];
	psi_i = new std::vector<int>[length+1];
	for (int i=1; i<=length; i++) {
		bppm[i] = new double[length+1];
		psi[i] = new double[length+1];
		for (int j=1; j<=length; j++) {
			bppm[i][j] = psi[i][j] = 0.0;
		}
	}
}

RNA::~RNA() {
	for (int i=1; i<=length; i++) {
		delete[] bppm[i];
		delete[] psi[i];
	}
	delete[] bppm;
	delete[] psi;
	delete[] psi_i;
}

// reads postscript file (output of RNAplfold -p -noLP)
RNA RNA::read_RNAfold_file(const char* filename) {
	const double minread = sqrt(0.0001); 
	std::ifstream in(filename);
	std::string sequence;
	char line[1024];
	while (in.getline(line, 1024)) {
		if (strstr(line, "/sequence")!=NULL) {
			for (;in.getline(line, 1024);) {
				char *p = line;
				for (; *p!='\0' && *p!=')'; ++p) {
					if (*p=='A' || *p=='C' || *p=='G' || *p=='U' || *p=='T') {
						sequence.push_back(*p);
					}
				}
				if (*p==')') break;
			}
			break;
		}
	}
	RNA rna(sequence);
	while (in.getline(line, 1024)) {
		if (*line=='%' || *line=='/') continue;
		if (strstr(line, "ubox")!=NULL) {
			int i, j;
			double p;
			sscanf(line, "%d %d %lf", &i, &j, &p);
			if (i>=1 && i<=rna.length && j>=1 && j<=rna.length && p>minread) {
				rna.bppm[i][j] = p*p;
				rna.psi[i][j] = std::max(log(p*rna.length*2) / log(rna.length*2), 0.0);
				if (rna.psi[i][j]>0.0) {
					rna.psi_i[i].push_back(j);
				}
			}
		}
	}
	in.close();
	return rna;
}

RNA RNA::fold_RNA(const char* sequence) {
	RNA rna(sequence);
	fill_bppm_RNAfold(sequence, rna.length, rna.bppm);
	for (int i=1; i<=rna.length; i++) {
		for (int j=i; j<=rna.length; j++) {
			rna.psi[i][j] = std::max(log(sqrt(rna.bppm[i][j])*rna.length*2) / log(rna.length*2), 0.0);
			if (rna.psi[i][j]>0.0) {
				rna.psi_i[i].push_back(j);
			}
		}
	}
	return rna;
}
