#ifndef INCLUDE_RNA_HPP
#define INCLUDE_RNA_HPP

#include <string>
#include <vector>


/* Used to access the RNA sequences, their base pair probability matrices, and ther psi-matrix
 */
class RNA {
	private:
		// private constructor
		RNA(const std::string &sequence);

	public:
		// length of this RNA
		const int length;
		// sequence of this RNA
		const std::string sequence;
		// base pair probability matrix, 0 <= p <= 1
		double** bppm;
		// psi (= logarithm of bppm)
		double** psi;
		// indices of positive entries in psi
		std::vector<int> *psi_i;
		// destructor
		~RNA();
		// creates an RNA object from a ps file
		static RNA read_RNAfold_file(const char* filename);
		// calls pf_fold() from Vienna RNA package
		static RNA fold_RNA(const char* sequence);

};


#endif
