#ifndef INCLUDE_SIGNIFICANT_BASE_PAIRS_HPP
#define INCLUDE_SIGNIFICANT_BASE_PAIRS_HPP

#include "rna.hpp"

struct significant_base_pairs {
	// the length of the first RNA
	const int N1;

	// the length of the second RNA
	const int N2;

	// bp[i][k]==true iff \exists j,l : (i,j) and (k,l) base pairs
	bool **bp;

	// given j, contains a list of i's with a.bppm[i][j]>=p_min
	std::vector<int> *bplist_a;

	// given l, contains a list of k's with a.bppm[k][l]>=p_min
	std::vector<int> *bplist_b;

	significant_base_pairs(const RNA &a, const RNA &b, const double p_min) : N1(a.length), N2(b.length) {
		bp = new bool*[N1+1];
		for (int i=1; i<=N1; i++) {
			bp[i] = new bool[N2+1];
			for (int k=1; k<=N2; k++) {
				bp[i][k] = false;
			}
		}
		for (int i=1; i<=N1; i++) {
			for (int j=1; j<=N1; j++) {
				if (a.bppm[i][j]>=p_min) {
				//if (a.psi[i][j]>0) {
					for (int k=1; k<=N2; k++) {
						for (int l=1; l<=N2; l++) {
							//if (b.bppm[k][l]>=p_min) {
							if (b.psi[k][l]>0) {
								bp[i][k] = true;
							}
						}
					}
				}
			}
		}
		bplist_a = new std::vector<int>[N1+1];
		for (int j=1; j<=N1; j++) {
			for (int i=1; i<j; i++) {
				if (a.bppm[i][j]>=p_min) {
				//if (a.psi[i][j]>0) {
					bplist_a[j].push_back(i);
				}
			}
		}
		bplist_b = new std::vector<int>[N2+1];
		for (int l=1; l<=N2; l++) {
			for (int k=1; k<l; k++) {
				if (b.bppm[k][l]>=p_min) {
				//if (b.psi[k][l]>0) {
					bplist_b[l].push_back(k);
				}
			}
		}
	}
	~significant_base_pairs() {
		delete[] bplist_a;
		delete[] bplist_b;
		for (int i=1; i<=N1; i++) delete[] bp[i];
		delete[] bp;
	}
	bool operator()(int i, int k) {
		return bp[i][k];
	}
};

#endif
