#ifndef INCLUDED_SPARSEMATRIX_H
#define INCLUDED_SPARSEMATRIX_H

/* 
 * This class is a wrapper for std::unordered_map and is indexed by four size_t values. 
 */

// Workaround to use <unordered_map> on MAC XCode
#ifndef IMPORT_TR1_UNORDERED_MAP
	#include <unordered_map>
#else
	#include <tr1/unordered_map>
	namespace std {
		using namespace tr1;
	}
#endif

template <class T>
class sparsematrix {
	private:
		struct quadruple {
			size_t i, j, k, l;
			quadruple() : i(0), j(0), k(0), l(0) {}
			quadruple(size_t i, size_t j, size_t k, size_t l) : i(i), j(j), k(k), l(l) {}
			quadruple(const quadruple &q) : i(q.i), j(q.j), k(q.k), l(q.l) {}
			inline bool operator==(const quadruple &q) const {
				return i==q.i && j==q.j && k==q.k && l==q.l;
			}
		};

		struct hash_quadruple {
			size_t operator()(const quadruple &q) const {
				//return std::hash<size_t>()(q.i) ^ std::hash<size_t>()(q.j) ^ 
				//		std::hash<size_t>()(q.k) ^ std::hash<size_t>()(q.l);
				return (std::hash<size_t>()(q.i) << 30) ^ (std::hash<size_t>()(q.j) << 20) 
						^ (std::hash<size_t>()(q.k) << 10) ^ std::hash<size_t>()(q.l);
			}
		};

		std::unordered_map<quadruple, T, hash_quadruple> map;
	public:

		typedef typename std::unordered_map<quadruple, T, hash_quadruple>::iterator iterator;
		typedef typename std::unordered_map<quadruple, T, hash_quadruple>::const_iterator const_iterator;

		T& operator()(size_t i, size_t j, size_t k, size_t l) {
			return map[quadruple(i,j,k,l)];
		}

		iterator begin() { return map.begin(); }
		iterator end() { return map.end(); }
		const_iterator begin() const { return map.begin(); }
		const_iterator end() const { return map.end(); }
		const_iterator cbegin() const { return map.cbegin(); }
		const_iterator cend() const { return map.cend(); }
		size_t size() const { return map.size(); }
};

template <class T>
inline std::ostream& operator<<(std::ostream &out, const sparsematrix<T> &s) {
	for (typename sparsematrix<T>::const_iterator it=s.begin(); it!=s.end(); ++it) {
		out << "(" << it->first.i << "," << it->first.j << "," << it->first.k << "," << it->first.l << ") = " << it->second << std::endl;
	}
}

#endif
